@@ -30,6 +30,9 @@
 #include "butil/containers/doubly_buffered_data.h"
 #include "brpc/describable.h"
 #include "brpc/socket.h"
+#include "brpc/socket_map.h"
+#include "brpc/global.h"
+#include "brpc/details/load_balancer_with_naming.h"
 #include "butil/strings/string_number_conversions.h"
 #include "brpc/excluded_servers.h" 
 #include "brpc/policy/weighted_round_robin_load_balancer.h"
@@ -556,25 +559,28 @@ TEST_F(LoadBalancerTest, consistent_hashing) {
             "10.36.150.32:8833", 
             "10.92.149.48:8833", 
             "10.42.122.201:8833",
+            "[2408:871a:2100:3:0:ff:b025:348d]:8833",
+            "unix:test.sock",
     };
     for (size_t round = 0; round < ARRAY_SIZE(hashs); ++round) {
         brpc::policy::ConsistentHashingLoadBalancer chlb(hash_type[round]);
         std::vector<brpc::ServerId> ids;
         std::vector<butil::EndPoint> addrs;
-        for (int j = 0;j < 5; ++j) 
-        for (int i = 0; i < 5; ++i) {
-            const char *addr = servers[i];
-            //snprintf(addr, sizeof(addr), "192.168.1.%d:8080", i);
-            butil::EndPoint dummy;
-            ASSERT_EQ(0, str2endpoint(addr, &dummy));
-            brpc::ServerId id(8888);
-            brpc::SocketOptions options;
-            options.remote_side = dummy;
-            options.user = new SaveRecycle;
-            ASSERT_EQ(0, brpc::Socket::Create(options, &id.id));
-            ids.push_back(id);
-            addrs.push_back(dummy);
-            chlb.AddServer(id);
+        for (int j = 0;j < 5; ++j) {
+            for (size_t i = 0; i < ARRAY_SIZE(servers); ++i) {
+                const char *addr = servers[i];
+                //snprintf(addr, sizeof(addr), "192.168.1.%d:8080", i);
+                butil::EndPoint dummy;
+                ASSERT_EQ(0, str2endpoint(addr, &dummy));
+                brpc::ServerId id(8888);
+                brpc::SocketOptions options;
+                options.remote_side = dummy;
+                options.user = new SaveRecycle;
+                ASSERT_EQ(0, brpc::Socket::Create(options, &id.id));
+                ids.push_back(id);
+                addrs.push_back(dummy);
+                chlb.AddServer(id);
+            }
         }
         std::cout << chlb;
         for (int i = 0; i < 5; ++i) {
@@ -1081,4 +1087,20 @@ TEST_F(LoadBalancerTest, revived_from_all_failed_intergrated) {
     ASSERT_EQ(0, num_failed.load(butil::memory_order_relaxed));
 }
 
+TEST_F(LoadBalancerTest, la_selection_too_long) {
+    brpc::GlobalInitializeOrDie();
+    brpc::LoadBalancerWithNaming lb;
+    CHECK_EQ(0, lb.Init("list://127.0.0.1:8888", "la", nullptr, nullptr)); 
+    char addr[] = "127.0.0.1:8888";
+    butil::EndPoint ep;
+    ASSERT_EQ(0, str2endpoint(addr, &ep));
+    brpc::SocketId id;
+    ASSERT_EQ(0, brpc::SocketMapFind(brpc::SocketMapKey(ep), &id));
+    ASSERT_EQ(0, brpc::Socket::SetFailed(id));
+    brpc::LoadBalancer::SelectIn in = { 0, false, false, 0u, nullptr };
+    brpc::SocketUniquePtr ptr;
+    brpc::LoadBalancer::SelectOut out(&ptr);
+    ASSERT_EQ(EHOSTDOWN, lb.SelectServer(in, &out));
+}
+
 } //namespace
